<?php
namespace nur\sery\sys;

/**
 * Class content: gestionnaire de contenu (statique ou dynamique)
 */
class content {
  /**
   * applatir $contents, et retourner une liste de valeurs scalaires:
   * - les instances de IContent et IPrintable sont remplacées par leur contenu
   * - les autres objets sont transformés en chaine de caractère avec strval()
   */
  static function flatten($contents, ?array &$dest=null): array {
    if ($dest === null) $dest = [];
    if ($contents === null) return $dest;
    if ($contents instanceof IContent) {
      $contents = $contents->getContent();
    } elseif ($contents instanceof IPrintable) {
      ob_start(null, 0, PHP_OUTPUT_HANDLER_STDFLAGS ^ PHP_OUTPUT_HANDLER_FLUSHABLE);
      $contents->print();
      $dest[] = ob_get_clean();
      return $dest;
    } elseif (!is_iterable($contents)) {
      $dest[] = $contents;
      return $dest;
    }
    foreach ($contents as $value) {
      self::flatten($value, $dest);
    }
    return $dest;
  }

  /**
   * résoudre tout le contenu statique: il ne restera plus qu'une liste de
   * chaines ou d'instances de IContent, IPrintable ou des objets quelconques
   */
  static function resolve_static($contents, $dest_class=null, bool $quote=true, ?array &$dest=null): array {
    if ($dest === null) $dest = [];
    if ($contents === null) return $dest;
    if (is_string($contents)) {
      if ($quote) $contents = htmlspecialchars($contents);
      $dest[] = $contents;
      return $dest;
    } elseif ($contents instanceof IContent) {
      $dest[] = $contents;
      return $dest;
    } elseif ($contents instanceof IPrintable) {
      $dest[] = $contents;
      return $dest;
    } elseif (!is_iterable($contents)) {
      $dest[] = $contents;
      return $dest;
    }
    foreach ($contents as $content) {
      if ($content === null) continue;
      elseif (is_string($content)) $dest[] = $content;
      elseif ($contents instanceof IContent) $dest[] = $content;
      elseif ($contents instanceof IPrintable) $dest[] = $content;
      elseif (!is_array($content)) $dest[] = $content;
      else {
        # ici, $content est soit:
        # - un appel de fonction ou méthode statique d'une des formes suivantes:
        #   - [class, method, ...args]
        #   - [null, method, ...args]
        #   - [[class, method], ...args]
        #   - [[null, method], ...args]
        # - ou une définition d'objet d'une des formes suivantes
        #   - [class, ...args]
        #   - [[class], ...args]
        $class = false;
        if (array_key_exists(0, $content) && is_array($content[0])) {
          # $content est de la forme [func_or_class, ...args]
          $func = $content[0];
          $args = array_slice($content, 1);
          if (func::is_class($func)) {
            $class = true;
            func::fix_class_args($func, $args);
          } else {
            func::ensure_func($func, $dest_class, $args);
          }
        } else {
          # $content est de la forme $func
          $func = $content;
          if (func::is_class($func)) {
            $class = true;
            func::fix_class_args($func, $args);
          } else {
            func::ensure_func($func, $dest_class, $args);
          }
        }
        # chaque argument est un contenu statique
        foreach ($args as &$arg) {
          $arg = self::resolve_static($arg, $dest_class, false);
          $arg = self::flatten($arg);
          switch (count($arg)) {
          case 0: $arg = null; break;
          case 1: $arg = $arg[0]; break;
          default: $arg = implode("", $arg);
          }
        }; unset($arg);
        # puis appeler la fonction
        if ($class) $content = func::cons($func, ...$args);
        else $content = func::call($func, ...$args);
        $dest[] = $content;
      }
    }
  }
}
